@@ -17,6 +17,7 @@ from awkward._nplikes.numpy_like import IndexType, NumpyMetadata
 from awkward._nplikes.placeholder import PlaceholderArray
 from awkward._nplikes.shape import ShapeItem, unknown_length
 from awkward._nplikes.typetracer import MaybeNone, TypeTracer
+from awkward._nplikes.virtual import VirtualArray, materialize_if_virtual
 from awkward._regularize import is_integer, is_integer_like
 from awkward._slicing import NO_HEAD
 from awkward._typing import (
@@ -481,6 +482,15 @@ class BitMaskedArray(BitMaskedMeta[Content], Content):
             return True
         return self._content._is_getitem_at_placeholder()
 
+    def _is_getitem_at_virtual(self) -> bool:
+        is_virtual = (
+            isinstance(self._mask.data, VirtualArray)
+            and not self._mask.data.is_materialized
+        )
+        if is_virtual:
+            return True
+        return self._content._is_getitem_at_virtual()
+
     def _getitem_at(self, where: IndexType):
         if not self._backend.nplike.known_data:
             self._touch_data(recursive=False)
@@ -702,12 +712,11 @@ class BitMaskedArray(BitMaskedMeta[Content], Content):
         cp = Cupy.instance()._module
 
         assert mask is None  # this class has its own mask
+        (mask,) = materialize_if_virtual(self._mask.data)
         if not self.lsb_order:
-            m = cp.flip(
-                cp.packbits(cp.flip(cp.unpackbits(cp.asarray(self._mask.data))))
-            )
+            m = cp.flip(cp.packbits(cp.flip(cp.unpackbits(cp.asarray(mask)))))
         else:
-            m = self._mask.data
+            m = mask
 
         if m.nbytes % 64:
             m = cp.resize(m, ((m.nbytes // 64) + 1) * 64)
@@ -809,7 +818,7 @@ class BitMaskedArray(BitMaskedMeta[Content], Content):
             )
 
         else:
-            excess_length = int(math.ceil(self._length / 8.0))
+            excess_length = math.ceil(self._length / 8.0)
             if (
                 self._mask.length is not unknown_length
                 and self._mask.length == excess_length
@@ -864,6 +873,26 @@ class BitMaskedArray(BitMaskedMeta[Content], Content):
             parameters=self._parameters,
         )
 
+    def _materialize(self) -> Self:
+        content = self._content.materialize()
+        mask = self._mask.materialize()
+        return BitMaskedArray(
+            mask,
+            content,
+            valid_when=self._valid_when,
+            length=len(self),
+            lsb_order=self._lsb_order,
+            parameters=self._parameters,
+        )
+
+    @property
+    def _is_all_materialized(self) -> bool:
+        return self._content.is_all_materialized and self._mask.is_all_materialized
+
+    @property
+    def _is_any_materialized(self) -> bool:
+        return self._content.is_any_materialized or self._mask.is_any_materialized
+
     def _is_equal_to(
         self, other: Self, index_dtype: bool, numpyarray: bool, all_parameters: bool
     ) -> bool:
