@@ -15,6 +15,7 @@ from awkward._nplikes.numpy_like import IndexType, NumpyMetadata
 from awkward._nplikes.placeholder import PlaceholderArray
 from awkward._nplikes.shape import ShapeItem, unknown_length
 from awkward._nplikes.typetracer import MaybeNone, TypeTracer
+from awkward._nplikes.virtual import VirtualArray, materialize_if_virtual
 from awkward._parameters import (
     parameters_intersect,
     parameters_union,
@@ -314,6 +315,15 @@ class IndexedOptionArray(IndexedOptionMeta[Content], Content):
             return True
         return self._content._is_getitem_at_placeholder()
 
+    def _is_getitem_at_virtual(self) -> bool:
+        is_virtual = (
+            isinstance(self._index.data, VirtualArray)
+            and not self._index.data.is_materialized
+        )
+        if is_virtual:
+            return True
+        return self._content._is_getitem_at_virtual()
+
     def _getitem_at(self, where: IndexType):
         if not self._backend.nplike.known_data:
             self._touch_data(recursive=False)
@@ -1562,7 +1572,7 @@ class IndexedOptionArray(IndexedOptionMeta[Content], Content):
         length: int,
         options: ToArrowOptions,
     ):
-        index = numpy.asarray(self._index.data, copy=True)
+        (index,) = materialize_if_virtual(numpy.asarray(self._index.data, copy=True))
         this_validbytes = self.mask_as_bool(valid_when=True)
         index[~this_validbytes] = 0
 
@@ -1660,10 +1670,11 @@ class IndexedOptionArray(IndexedOptionMeta[Content], Content):
             npselect = npindex >= 0
             if self._backend.index_nplike.any(npselect):
                 indexmin = self._backend.index_nplike.min(npindex[npselect])
+                indexmax = self._backend.index_nplike.max(npindex)
                 index = ak.index.Index(
                     npindex - indexmin, nplike=self._backend.index_nplike
                 )
-                content = self._content[indexmin : npindex.max() + 1]
+                content = self._content[indexmin : indexmax + 1]
             else:
                 index, content = self._index, self._content
         else:
@@ -1751,7 +1762,7 @@ class IndexedOptionArray(IndexedOptionMeta[Content], Content):
         if out is not None:
             return out
 
-        index = self._index.raw(numpy)
+        (index,) = materialize_if_virtual(self._index.raw(numpy))
         not_missing = index >= 0
         content = ak.to_backend(self._content, "cpu", highlevel=False)
         nextcontent = content._carry(ak.index.Index(index[not_missing]), False)
@@ -1768,6 +1779,19 @@ class IndexedOptionArray(IndexedOptionMeta[Content], Content):
         index = self._index.to_nplike(backend.index_nplike)
         return IndexedOptionArray(index, content, parameters=self._parameters)
 
+    def _materialize(self) -> Self:
+        content = self._content.materialize()
+        index = self._index.materialize()
+        return IndexedOptionArray(index, content, parameters=self._parameters)
+
+    @property
+    def _is_all_materialized(self) -> bool:
+        return self._content.is_all_materialized and self._index.is_all_materialized
+
+    @property
+    def _is_any_materialized(self) -> bool:
+        return self._content.is_any_materialized or self._index.is_any_materialized
+
     def _is_equal_to(
         self, other: Self, index_dtype: bool, numpyarray: bool, all_parameters: bool
     ) -> bool:
