@@ -9,10 +9,12 @@ from awkward._backends.dispatch import regularize_backend
 from awkward._dispatch import high_level_function
 from awkward._layout import wrap_layout
 from awkward._nplikes.array_like import ArrayLike
+from awkward._nplikes.jax import Jax
 from awkward._nplikes.numpy import Numpy
 from awkward._nplikes.numpy_like import NumpyLike, NumpyMetadata
 from awkward._nplikes.placeholder import PlaceholderArray
 from awkward._nplikes.shape import ShapeItem, unknown_length
+from awkward._nplikes.virtual import VirtualArray
 from awkward._regularize import is_integer
 from awkward.forms.form import index_to_dtype, regularize_buffer_key
 
@@ -82,6 +84,9 @@ def from_buffers(
     #ak.forms.NumpyForm.
     If the values of `container` are recognised as arrays by the given backend,
     a view over their existing data will be used, where possible.
+    The `container` values are allowed to be callables with no arguments.
+    If that's the case, they will be turned into `VirtualArray` buffers whose generator
+    function is the callable and is used to materialize the buffer when required.
 
     The `buffer_key` should be the same as the one used in #ak.to_buffers.
 
@@ -162,10 +167,21 @@ def _from_buffer(
     byteorder: str,
     field_path: tuple,
 ) -> ArrayLike:
+    if callable(buffer):
+        # This is the case for VirtualArrays
+        # We use recursion here to pass down the from_buffer and byteorder transformations to the generator
+        return VirtualArray(
+            nplike=nplike,
+            shape=(count,),
+            dtype=dtype,
+            generator=lambda: _from_buffer(
+                nplike, buffer(), dtype, count, byteorder, field_path
+            ),
+        )
     # Unknown-length information implies that we didn't load shape-buffers (offsets, etc)
     # for the parent of this node. Thus, this node and its children *must* only
     # contain placeholders
-    if count is unknown_length:
+    elif count is unknown_length:
         # We may actually have a known buffer here, but as we do not know the length,
         # we cannot safely trim it. Thus, introduce a placeholder anyway
         return PlaceholderArray(nplike, (unknown_length,), dtype, field_path)
@@ -176,7 +192,8 @@ def _from_buffer(
         return PlaceholderArray(nplike, (count,), dtype, field_path)
     elif isinstance(buffer, PlaceholderArray) or nplike.is_own_array(buffer):
         # Require 1D buffers
-        array = nplike.reshape(buffer.view(dtype), shape=(-1,), copy=False)
+        copy = None if isinstance(nplike, Jax) else False  # Jax can not avoid this
+        array = nplike.reshape(buffer.view(dtype), shape=(-1,), copy=copy)
 
         if array.size < count:
             raise TypeError(
@@ -238,7 +255,7 @@ def _reconstitute(
         if length is unknown_length:
             next_length = unknown_length
         else:
-            next_length = int(math.ceil(length / 8.0))
+            next_length = math.ceil(length / 8.0)
         mask = _from_buffer(
             backend.index_nplike,
             raw_array,
@@ -317,6 +334,9 @@ def _reconstitute(
             next_length = (
                 0 if len(index) == 0 else max(0, backend.index_nplike.max(index) + 1)
             )
+            # free memory
+            if isinstance(index, VirtualArray):
+                index.dematerialize()
         content = _reconstitute(
             form.content,
             next_length,
@@ -357,6 +377,9 @@ def _reconstitute(
                     backend.index_nplike.max(index) + 1
                 )
             )
+            # free memory
+            if isinstance(index, VirtualArray):
+                index.dematerialize()
         content = _reconstitute(
             form.content,
             next_length,
@@ -403,6 +426,11 @@ def _reconstitute(
             next_length = (
                 0 if len(starts) == 0 else backend.index_nplike.max(reduced_stops)
             )
+            # free memory
+            if isinstance(starts, VirtualArray):
+                starts.dematerialize()
+            if isinstance(stops, VirtualArray):
+                stops.dematerialize()
         content = _reconstitute(
             form.content,
             next_length,
@@ -435,6 +463,9 @@ def _reconstitute(
             next_length = unknown_length
         else:
             next_length = 0 if len(offsets) == 1 else offsets[-1]
+            # free memory
+            if isinstance(offsets, VirtualArray):
+                offsets.dematerialize()
         content = _reconstitute(
             form.content,
             next_length,
@@ -520,6 +551,11 @@ def _reconstitute(
                     lengths.append(0)
                 else:
                     lengths.append(backend.index_nplike.max(selected_index) + 1)
+            # free memory
+            if isinstance(index, VirtualArray):
+                index.dematerialize()
+            if isinstance(tags, VirtualArray):
+                tags.dematerialize()
         contents = [
             _reconstitute(
                 content,
